#!/usr/bin/python
# -*- Mode: Python; python-indent: 8; indent-tabs-mode: t -*-

import sys, os #, errno
import subprocess
import re

from preup.script_api import *





set_component("samba")



set_component("samba")
"""Preupgrade assistant performs system upgradability assessment
and gathers information required for successful operating system upgrade.
Copyright (C) 2013 Red Hat Inc.
Frantisek Kluknavsky <fkluknav@redhat.com>

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>."""
check_applies_to (check_applies="samba")
set_component("samba")
#END GENERATED SECTION
# exit functions are exit_{pass,not_applicable, fixed, fail, etc.}
# logging functions are log_{error, warning, info, etc.}
# for logging in-place risk use functions log_{extreme, high, medium, slight}_risk

def samba_dirs():
	"""Parse smb.conf and report shared directories."""

	def wrong_section(name):
		"""Check whether a name is one of special section names which we want to ignore"""
		wrong = ("global", "homes", "printers")
		for i in wrong:
			if i == name:
				return True
		return False

	def lsselinux(path):
		try:
			raw_diff = os.tmpfile();
			return_code = subprocess.Popen(["ls", "-lZd", path],
			bufsize=0,
			executable=None,
			stdin=None,
			stdout=raw_diff,
			stderr=subprocess.STDOUT,
			preexec_fn=None,
			close_fds=True,
			shell=False,
			cwd=None,
			env=None,
			universal_newlines=False,
			startupinfo=None,
			creationflags=0).wait()
		except:
			raw_diff.seek(0)
			log_error("Error while invoking ls")
			log_error(raw_diff.read())
			raise
		raw_diff.seek(0)
		if return_code:
			log_warning(raw_diff.read())
			return False
		#log_high_risk(raw_diff.read())
		return raw_diff.read()


	found = [] #return value - found paths to shared directories
	section = None
	multiline_re = re.compile(r"(.*)\\\s*$")
	comment_re = re.compile(r"\s*[#;].*$")
	section_re = re.compile(r"\s*\[(.*)\]\s*$")
	value_re = re.compile(r"\s*(.*?)\s*=\s*(.*?)\s*$")
	white_re = re.compile(r"\s*$")
	selinux_re = re.compile(r"(\s*[^\s]*){3}\s*(.*?)\s*$")
	with open(r"/etc/samba/smb.conf", "r") as smbconf_f:
		smbconf=smbconf_f.readlines()
	line_buffer = ""
	for line in smbconf:
		if comment_re.match(line):
			line = ""
		multiline_match = multiline_re.match(line)
		if multiline_match:
			line_buffer += multiline_match.group(1)
			continue
		line_buffer += line
		match = section_re.match(line_buffer)
		if match:
			section = match.group(1)
		else:
			match = value_re.match(line_buffer)
			if match:
				if wrong_section(section):
					pass
				else:
					if match.group(1) == "path":
						found.append(match.group(2))
						log_medium_risk("Found directory " + match.group(2) + " in section [" + section + "] in smb.conf")
						lss = lsselinux(match.group(2))
						if lss:
							log_medium_risk(selinux_re.match(lss).group(2))
			else:
				match = white_re.match(line_buffer)
				if not match:
					log_error("Misunderstood "+line_buffer)
					exit_error()
		line_buffer = ""
	return found

if __name__ == "__main__":
	if os.geteuid() != 0:
		sys.stdout.write("Need to be root.\n")
		log_slight_risk("The script needs to be run under root account")
		exit_error()
	if samba_dirs():
		exit_informational()
	else:
		exit_pass()



#!/usr/bin/python
# -*- Mode: Python; python-indent: 8; indent-tabs-mode: t -*-
import sys, os, errno, re, shutil
import datetime

from preup.script_api import *

"""Preupgrade assistant performs system upgradability assessment
and gathers information required for successful operating system upgrade.
Copyright (C) 2013 Red Hat Inc.
Valentina Mukhamedzhanova <vmukhame@redhat.com>

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>."""
check_applies_to (check_applies="yum")
set_component("yum")
#END GENERATED SECTION
component = "yum"


def main():
    result ='clean'

    config_file = '/etc/yum.conf'
    upg_config_file = os.path.join(VALUE_TMP_PREUPGRADE, 'cleanconf', config_file[1:])
    upg_config_dir = os.path.dirname(upg_config_file)
    try:
        os.makedirs(upg_config_dir)
    except OSError:
        if not os.path.isdir(upg_config_dir):
            raise
    shutil.copyfile(config_file, upg_config_file)

    with open(VALUE_RPM_RHSIGNED, "r") as f:
        lines = f.readlines()
    rpms = [line.split('\t')[0] for line in lines]

    if 'yum-plugin-downloadonly' in rpms:
        solution_file("In RHEL 7 functionality of yum-plugin-downloadonly is a part of yum core.\n\n")
    if 'yum-plugin-security' in rpms:
        solution_file("In RHEL 7 functionality of yum-plugin-security is a part of yum core.\n\n")
    if 'yum-presto' in rpms:
        solution_file("In RHEL 7 functionality of yum-presto is a part of yum core and --disablepresto option is no longer supported, please make sure none of your scripts are relying on it.\n")
        with open('/etc/yum/pluginconf.d/presto.conf', 'r') as presto_conf:
            for line in presto_conf.readlines():
                line = line.strip()
                if (line.startswith('keepdeltas') or line.startswith('minimum_percentage')) and result=='clean':
                    solution_file('In RHEL 7 deltarpm configuration options have changed and moved from /etc/yum/pluginconf.d/presto.conf to /etc/yum.conf.\n')
                    result = 'fixed'
                if line.startswith('keepdeltas'):
                    solution_file("Your /etc/yum/pluginconf.d/presto.conf file contains 'keepdeltas' option, which is not supported in RHEL 7. This option will not be copied to your /etc/yum.conf\n")
                elif line.startswith('minimum_percentage'):
                    solution_file("Your /etc/yum/pluginconf.d/presto.conf file contains 'minimum_percentage' option, which is not supported in RHEL 7, but you can use 'deltarpm_percentage' instead in /etc/yum.conf.\n")
                    deltarpm_percentage = None
                    try:
                        deltarpm_percentage = re.match(r'minimum_percentage\s*=\s*(\d+)$', line).groups()[0]
                    except:
                        solution_file("Cannot parse minimum_percentage value: '%s'. Please review it and manually add 'deltarpm_percentage' option to /etc/yum.conf.\n" % line)
                    if not deltarpm_percentage is None:
                        solution_file("Current value of 'minumum_percentage' is %s. This value will be copied to 'deltarpm_percentage' option in /etc/yum.conf.\n" % deltarpm_percentage)
                        with open(upg_config_file, 'a') as dst:
                           dst.write("\n# This value has been copied from 'minimum_percentage' option value from yum-presto config file.\ndeltarpm_percentage=%s\n" % deltarpm_percentage)
        solution_file("\n")

    solution_file("After the upgrade it will be impossible to undo/redo/rollback to pre-upgrade yum transactions. Please run 'yum history new' after the upgrade to start a new history file.\n\n")
    solution_file("The way yum groups work has changed in RHEL 7. By default yum treats groups as objects now. Please refer to the documentation for more information.\n\n")

    return result


if __name__ == "__main__":
	set_component(component)
	result = main()
        if result == 'fixed':
            exit_fixed()
	exit_informational()




#!/usr/bin/python2
# -*- Mode: Python; python-indent: 8; indent-tabs-mode: t -*-
"""
"""

import os
import re
from collections import namedtuple
from preup.script_api import *


"""Preupgrade assistant performs system upgradability assessment
and gathers information required for successful operating system upgrade.
Copyright (C) 2013 Red Hat Inc.
Tomas Hozza <thozza@redhat.com>

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>."""
check_applies_to (check_applies="dnsmasq")
check_rpm_to (check_rpm="",check_bin="python")
set_component("dnsmasq")
#END GENERATED SECTION
# exit functions are exit_{pass,not_applicable, fixed, fail, etc.}
COMPONENT = "DNSMASQ"
# logging functions are log_{error, warning, info, etc.}
# for logging in-place risk use functions log_{extreme, high, medium, slight}_risk
ConfFile = namedtuple("ConfFile", ["path", "buffer"])
CONFIG_FILE = "/etc/dnsmasq.conf"
FILES_TO_CHECK = []

# Exit codes
EXIT_NOT_APPLICABLE = 0
EXIT_PASS = 1
EXIT_INFORMATIONAL = 2
EXIT_FIXED = 3
EXIT_FAIL = 4
EXIT_ERROR = 5


class SolutionText(object):
    """
    class for handling construction of solution text.
    """
    def __init__(self):
        self.header = """Some issues have been found in your DNSMASQ configuration.
Use following solutions to fix them:"""
        self.tail = """For more information, please see the DNSMASQ manual page
'man 8 dnsmasq'."""
        self.solutions = []

    def add_solution(self, solution=""):
        if solution:
            self.solutions.append(solution)

    def get_text(self):
        text = self.header + "\n\n\n"
        for solution in self.solutions:
            text += solution + "\n\n\n"
        text += self.tail
        return text


# object used for creating solution text
sol_text = SolutionText()


#######################################################
### CONFIGURATION CHECKS PART - BEGIN
#######################################################


CONFIG_CHECKS = []


def register_check(check):
    """
    Function decorator that adds configuration check into list of checks.
    """
    CONFIG_CHECKS.append(check)
    return check


def run_checks(files_to_check):
    """
    Runs all available checks on files loaded into files_to_check list.
    """
    gl_result = EXIT_PASS

    for check in CONFIG_CHECKS:
        log_info("Running check: \"" + check.__name__ + "\"")
        for fpath, buff in FILES_TO_CHECK:
            log_info("checking: \"" + fpath + "\"")
            result = check(fpath, buff)
            if result > gl_result:
                gl_result = result

    return gl_result


@register_check
def check_interface_labels(file_path, buff):
    """
    Handle IPv4 interface-address labels in Linux. These are
    often used to emulate the old IP-alias addresses. Before,
    using --interface=eth0 would service all the addresses of
    eth0, including ones configured as aliases, which appear
    in ifconfig as eth0:0. Now, only addresses with the label
    eth0 are active. This is not backwards compatible: if you
    want to continue to bind the aliases too, you need to add
    eg. --interface=eth0:0 to the config.
    """

    status = EXIT_PASS

    interface_pattern_str = "interface=(.+)"
    interface_pattern = re.compile(interface_pattern_str)
    interface_stat_iter = interface_pattern.finditer(buff)

    for statement in interface_stat_iter:
        log_medium_risk("Found '" + statement.group(0) + "' option in \"" + file_path + "\".")
        status = EXIT_FAIL

    if status == EXIT_FAIL:
        sol_text.add_solution(
"""'interface' option with defined interface name:
--------------------------------------------------
Previously, dnsmasq configured with ex. '--interface=eth0' would bind
and listen on all addresses of the 'eth0' interface, including addresses
configured as aliases (which appear in ifconfig output as ex. 'eth0:0').
Now only addresses with the label 'eth0' are used (in other words, the
addresses configured as aliases for the interface are NOT used).
If you want dnsmasq to continue to listen on addresses configured as
asliases for the interface, you have to specify each alias in the
configuration using the 'interface' option (ex. '--interface=eth0:0').""")

    return status


@register_check
def check_dhcp_tags(file_path, buff):
    """
    Rationalised the DHCP tag system. Every configuration item
    which can set a tag does so by adding "set:<tag>" and
    every configuration item which is conditional on a tag is
    made so by "tag:<tag>". The NOT operator changes to '!',
    which is a bit more intuitive too. Dhcp-host directives
    can set more than one tag now. The old '#' NOT,
    "net:" prefix and no-prefixes are still honoured, so
    no existing config file needs to be changed, but
    the documentation and new-style config files should be
    much less confusing.
    """

    status = EXIT_PASS
    dhcp_tag_pattern_str = ".*=(.*?net:.+)"
    dhcp_tag_pattern = re.compile(dhcp_tag_pattern_str)
    dhcp_tag_iter = dhcp_tag_pattern.finditer(buff)

    for statement in dhcp_tag_iter:
        log_slight_risk("It looks like you are using DHCP tags ('" + statement.group(0) + "') in \"" + file_path + "\".")
        status = EXIT_INFORMATIONAL

    if status == EXIT_INFORMATIONAL:
        sol_text.add_solution(
"""Using DHCP tag system:
-------------------------
In new dnsmasq version, the way of configuring tags used in DHCP options
have changed. The old way is still still works, but it is advised to use
the new syntax for new configurations and if possible also for existing
ones. The new syntax in configuration options supprting tags is as follows:
- To set a tag, use 'set:<tag>' as the argument of the option.
- To match a tach, use 'tag:<tag>' as the argument of the option,
  (instead of old 'net:<tag>' way).
- As a NOT operator use '!' (instead of old '#').

dhcp-host option can now set more than one tag.""")

    return status


#######################################################
### CONFIGURATION CHECKS PART - END
#######################################################


def is_config_changed():
    """
    Checks if configuration files changed.
    """
    with open(VALUE_ALLCHANGED, "r") as f:
        files = f.read()
        for fpath, buff in FILES_TO_CHECK:
            found = re.findall(fpath, files)
            if found:
                return True
    return False


def return_with_code(code):
    if code == EXIT_FAIL:
        exit_fail()
    elif code == EXIT_FIXED:
        exit_fixed()
    elif code == EXIT_NOT_APPLICABLE:
        exit_not_applicable()
    elif code == EXIT_PASS:
        exit_pass()
    elif code == EXIT_ERROR:
        exit_error()
    elif code == EXIT_INFORMATIONAL:
        exit_informational()
    else:
        exit_unknown()


def remove_comments(lines):
    """
    Removes the following types of comments from passed string and returns it:
    # .*
    """
    string = ""
    for line in lines:
        tmp = line.strip()
        if tmp and not tmp.startswith("#"):
            string += tmp + "\n"
    return string


def is_file_loaded(path=""):
    """
    Checks if file with given 'path' is already loaded in FILES_TO_CHECK.
    """
    for f in FILES_TO_CHECK:
        if f.path == path:
            return True
    return False


def load_more_config():
    """
    Finds configuration files that are included in some configuration
    file, reads it, closes and adds into FILES_TO_CHECK list.
    """
    def load_config_file(path=""):
        """
        Load file with the given path if it is not already loaded.
        """
        if is_file_loaded(path):
            return
        try:
            f = open(path, 'r')
        except:
            log_error("Cannot open configuration file: \"" + path + "\"")
            exit_error()
        else:
            log_info("Loading configuration file \"" + path + "\"")
            filtered_string = remove_comments(f.readlines())
            f.close()
            FILES_TO_CHECK.append(ConfFile(buffer=filtered_string,
                                           path=path))


    def load_config_from_dir(path=""):
        """
        Walk recursively through the given dir and load all files.
        """
        def filter_files(path):
            if os.path.isfile(path):
                f = os.path.split(path)[1]
                if not f.endswith("~") and not f.startswith(".") and not (f.startswith("#") and f.endswith("#")):
                    return True
            return False

        log_info("Checking \"" + path + "\" for configuration files...")
        files = [ os.path.join(path, f) for f in os.listdir(path) if filter_files(os.path.join(path, f)) ]
        for f in files:
            load_config_file(os.path.join(path, f))


    pattern_conf_file = re.compile("conf-file\s*=\s*(.+)\s*")
    pattern_conf_dir = re.compile("conf-dir\s*=\s*(.+)\s*")

    # walk through all already loaded files
    for ch_file in FILES_TO_CHECK:
        # find all configuration files
        config_files = pattern_conf_file.findall(ch_file.buffer)
        for f in config_files:
            load_config_file(f)
        # find all condiguration directories
        config_dirs = pattern_conf_dir.findall(ch_file.buffer)
        for d in config_dirs:
            load_config_from_dir(d)


def load_main_config():
    """
    Loads main CONFIG_FILE.
    """
    try:
        f = open(CONFIG_FILE, 'r')
    except IOError:
        log_error(
            "Cannot open the configuration file: \"" + CONFIG_FILE + "\"")
        exit_error()
    else:
        log_info("Loading configuration file: \"" + CONFIG_FILE + "\"")
        filtered_string = remove_comments(f.readlines())
        f.close()
        FILES_TO_CHECK.append(ConfFile(buffer=filtered_string,
                                       path=CONFIG_FILE))


def main():
    load_main_config()
    load_more_config()
    # need to check also paths of included files
    if not is_config_changed():
        return_with_code(EXIT_NOT_APPLICABLE)
    result = run_checks(FILES_TO_CHECK)
    # if there was some issue, write a solution text
    if result > EXIT_PASS:
        solution_file(sol_text.get_text())
    return_with_code(result)


if __name__ == "__main__":
    set_component(COMPONENT)
    main()



#!/usr/bin/python2

import subprocess
import re
from preup.script_api import *


"""Preupgrade assistant performs system upgradability assessment
and gathers information required for successful operating system upgrade.
Copyright (C) 2013 Red Hat Inc.
Martin Kosek <mkosek@redhat.com>

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>."""
check_applies_to (check_applies="ipa-server")
check_rpm_to (check_rpm="",check_bin="python")
set_component("ipa-server")
#END GENERATED SECTION

try:
    from ipaserver.install import installutils
    from ipalib import api
    ipaserver_installed = True
except ImportError:
    ipaserver_installed = False

COMPONENT = 'ipa-server'

def main():
    if not ipaserver_installed:
        log_info("ipa-server package not installed")
        exit_not_applicable()

    if not installutils.is_ipa_configured():
        log_info("Identity Managament Server not configured")
        exit_not_applicable()

    log_extreme_risk("Identity Management Server cannot be upgraded in-place")
    exit_fail()

if __name__ == "__main__":
    set_component(COMPONENT)
    main()
    exit_pass()




#!/usr/bin/python
# -*- Mode: Python; python-indent: 8; indent-tabs-mode: t -*-

import sys, os #, errno
#import datetime
import re

from preup.script_api import *


"""Preupgrade assistant performs system upgradability assessment
and gathers information required for successful operating system upgrade.
Copyright (C) 2013 Red Hat Inc.
Frantisek Kluknavsky <fkluknav@redhat.com>

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>."""
check_applies_to (check_applies="filesystem")
set_component("filesystem")
#END GENERATED SECTION
component = "vcsrepos"
# exit functions are exit_{pass,not_applicable, fixed, fail, etc.}
# logging functions are log_{error, warning, info, etc.}
# for logging in-place risk use functions log_{extreme, high, medium, slight}_risk

def main():
	if os.geteuid() != 0:
		sys.stdout.write("Need to be root.\n")
		log_slight_risk("The script needs to be run under root account")
		exit_error()
	suffixes = ("/.git\n", "/.svn\n", "/.bzr\n", "/CVS\n")
	messages = {suffixes[0]:"GIT", suffixes[1]:"SUBVERSION", suffixes[2]:"BAZAAR", suffixes[3]:"CVS"}
	with open(VALUE_ALLMYFILES,"r") as allmyfiles:
		homefiles = filter(lambda x: x.startswith((r"/home/", r"/root")) and x.endswith(suffixes), allmyfiles.readlines())
	if homefiles:
		for suffix in suffixes:
			paths = filter(lambda x: x.endswith(suffix), homefiles)
			if paths:
				solution_file("Found " + messages[suffix] + " repositories:\n")
				for path in paths:
					if suffix == suffixes[0]: #git
						try:
							with open(path[:-1]+"/config", "r") as gitconfig:
								solution_file(path[:-len(suffix)]+"\n")
								for line in gitconfig:
									url = re.match(r"\s*url\s*=\s*(.*)", line)
									if url:
										addr = re.match(r"(ftp://|https?://)(.*)", url.group(1))
										if addr:
											solution_file("url = [link: " + url.group(1) + "]\n")
										else:
											solution_file(line)
						except IOError:
							pass
					elif suffix == suffixes[1]: #svn
						try:
							with open(path[:-1]+"/entries", "r") as config:
								solution_file(path[:-len(suffix)]+"\n")
								line = config.readlines()[4]
								if re.match(r"\s*(https?|ftp)://", line):
									solution_file("url = [link: " + line[:-1] + "]\n")
								else:
									solution_file("url = " + line)
						except IOError:
							pass
					elif suffix == suffixes[2]: #bzr
						try:
							with open(path[:-1]+"/branch/branch.conf", "r") as config:
								solution_file(path[:-len(suffix)]+"\n")
								for line in config:
									addr = re.match(r"\s*parent_location\s*=\s*(.*)", line)
									if addr:
										if re.match(r"(https?|ftp)://", addr.group(1)):
											solution_file("url = [link: " + addr.group(1) + "]\n")
										else:
											solution_file("url = " + addr.group(1) + "\n")
						except IOError:
							pass
					elif suffix == suffixes[3]: #cvs
						try:
							with open(path[:-1]+"/Root", "r") as config:
								solution_file(path[:-len(suffix)]+"\n")
								for line in config:
									solution_file(line)
						except IOError:
							pass
					else:
						solution_file(path[:-len(suffix)]+"\n")
				solution_file("\n")
	else:
		solution_file("VCS repositories not found.")


if __name__ == "__main__":
	set_component(component)
	main()
	exit_informational()



#!/usr/bin/python
# -*- Mode: Python; python-indent: 8; indent-tabs-mode: t -*-
"""
"""

import sys
import os
import re
from collections import namedtuple
from preup.script_api import *


"""Preupgrade assistant performs system upgradability assessment
and gathers information required for successful operating system upgrade.
Copyright (C) 2013 Red Hat Inc.
Tomas Hozza <thozza@redhat.com>

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>."""
check_applies_to (check_applies="bind")
check_rpm_to (check_rpm="",check_bin="python")
set_component("bind")
#END GENERATED SECTION
# exit functions are exit_{pass,not_applicable, fixed, fail, etc.}
COMPONENT = "BIND"
# logging functions are log_{error, warning, info, etc.}
# for logging in-place risk use functions log_{extreme, high, medium, slight}_risk
ConfFile = namedtuple("ConfFile", ["path", "buffer"])
CONFIG_FILE = "/etc/named.conf"
FILES_TO_CHECK = []

FIXED_CONFIGS = {}

# Exit codes
EXIT_NOT_APPLICABLE = 0
EXIT_PASS = 1
EXIT_INFORMATIONAL = 2
EXIT_FIXED = 3
EXIT_FAIL = 4
EXIT_ERROR = 5


class SolutionText(object):
    """
    class for handling construction of solution text.
    """
    def __init__(self):
        self.header = """Some issues have been found in your BIND9 configuration.
Use following solutions to fix them:"""
        self.tail = """For more information, please see the BIND9 Administrator Reference
Manual located in '/usr/share/doc/bind-9.9.4/Bv9ARM.pdf' and 'DNS Servers'
section of RHEL 7 Networking Guide."""
        self.solutions = []

    def add_solution(self, solution=""):
        if solution:
            self.solutions.append(solution)

    def get_text(self):
        text = self.header + "\n\n\n"
        for solution in self.solutions:
            text += solution + "\n\n\n"
        text += self.tail
        return text


# object used for creating solution text
sol_text = SolutionText()


#######################################################
### CONFIGURATION CHECKS PART - BEGIN
#######################################################


CONFIG_CHECKS = []


def register_check(check):
    """
    Function decorator that adds configuration check into list of checks.
    """
    CONFIG_CHECKS.append(check)
    return check


def run_checks(files_to_check):
    """
    Runs all available checks on files loaded into files_to_check list.
    """
    gl_result = EXIT_PASS

    for check in CONFIG_CHECKS:
        log_info("Running check: \"" + check.__name__ + "\"")
        for fpath, buff in FILES_TO_CHECK:
            log_info("checking: \"" + fpath + "\"")
            result = check(fpath, buff)
            if result > gl_result:
                gl_result = result

    log_info("Running check: \"check_empty_zones_complex\"")
    result = check_empty_zones_complex()
    if result > gl_result:
        gl_result = result
    
    log_info("Running check: \"check_default_runtime_dir\"")
    result = check_default_runtime_dir()
    if result > gl_result:
        gl_result = result

    return gl_result


@register_check
def check_tcp_listen_queue(file_path, buff):
    """
    3581.	[bug]		Changed the tcp-listen-queue default to 10. [RT #33029]

    Default and minimum value changed from 3 -> 10

    From bind-9.9.4 ARM:
    The listen queue depth. The default and minimum is 10. If the kernel supports the
    accept filter 'dataready' this also controls how many TCP connections that will be queued in
    kernel space waiting for some data before being passed to accept. Nonzero values less than 10
    will be silently raised. A value of 0 may also be used; on most platforms this sets the listen queue
    length to a system-defined default value.
    """
    pattern = re.compile("tcp-listen-queue\s*([0-9]+)\s*;")
    match_iter = pattern.finditer(buff)
    status = EXIT_PASS

    for match in match_iter:
        try:
            number = int(match.group(1))
        except ValueError:
            log_error("Value \"" + match.group(1) + "\" cannot be converted")
            return EXIT_ERROR
        # the new default and minimum value is "10"
        if number > 0 and number < 10:
            log_slight_risk("Found \"" + match.group(0) + "\" in \"" +
                            file_path + "\"")
            sol_text.add_solution(
"""'tcp-listen-queue' statement with value less than '10':
-------------------------------------------------------
The value specified in 'tcp-listen-queue' statement is less than '10'.
You should:
- Change your configuration to use at least value of '10'. BIND9
  will silently ignore value < '10' and use '10' instead.""")
            status = EXIT_INFORMATIONAL

    return status


@register_check
def check_zone_statistics(file_path, buff):
    """
    3501.	[func]   zone-statistics now takes three options: full,
                    terse, and none. "yes" and "no" are retained as
                    synonyms for full and terse, respectively. [RT #29165]

    Options changed, but are still compatible and can be used in new version.

    From bind-9.9.4 ARM:
    If full, the server will collect statistical data on all zones (unless specifically turned off
    on a per-zone basis by specifying zone-statistics terse or zone-statistics none in the zone state-
    ment). The default is terse, providing minimal statistics on zones (including name and current
    serial number, but not query type counters).

    For backward compatibility with earlier versions of BIND 9, the zone-statistics option can also
    accept yes or no, which have the same effect as full and terse, respectively.
    """
    pattern = re.compile("zone-statistics\s*(yes|no)\s*;")
    match_iter = pattern.finditer(buff)
    status = EXIT_PASS

    for match in match_iter:
        log_slight_risk("Found \"" + match.group(0) + "\" in \"" +
                        file_path + "\"")
        sol_text.add_solution(
"""'zone-statistics' arguments changed:
------------------------------------
Arguments of 'zone-statistics' option changed in new version of BIND9.
You should:
- Replace argument 'yes' with 'full', or replace argument 'no' with
  'terse'. Old options are still recognised by BIND9 and silently
  converted.""")
        status = EXIT_INFORMATIONAL

    return status


@register_check
def check_masterfile_format(file_path, buff):
    """
    3180.	[func]		Local copies of slave zones are now saved in raw
                            format by default, to improve startup performance.
                            'masterfile-format text;' can be used to override
                            the default, if desired. [RT #25867]

    The default format of saved slave zone changed from 'text' -> 'raw'

    From bind-9.9.4 ARM:
    masterfile-format Specifies the file format of zone files (see Section 6.3.7). The default value is text,
    which is the standard textual representation, except for slave zones, in which the default value
    is raw. Files in other formats than text are typically expected to be generated by the named-
    compilezone tool, or dumped by named.
    """
    pattern_zone_str = "zone\s+\"(.+?)\"(\s|.)*?{(\s|.)*?}"
    pattern_slave_str = "type\s+slave"
    pattern_mff_str = "masterfile-format"
    status = EXIT_PASS

    # find slave zones without masterfile-format statement
    pattern_zone = re.compile(pattern_zone_str)
    pattern_sl_zone = re.compile(pattern_slave_str)
    pattern_mff = re.compile(pattern_mff_str)
    pattern_zone_iter = pattern_zone.finditer(buff)

    for zone in pattern_zone_iter:
        slave_statement = pattern_sl_zone.search(zone.group(0))
        # if slave zone
        if slave_statement:
            mff_statement = pattern_mff.search(zone.group(0))
            # if no masterfile-format statement
            if not mff_statement:
                log_medium_risk("Found slave zone \"" + zone.group(1) + "\" in \"" +
                                file_path + "\" without \"masterfile-format\" statement.")
                status = EXIT_FAIL
    
    if status == EXIT_FAIL:
        sol_text.add_solution(
"""slave zone definition without 'masterfile-format' statement:
------------------------------------------------------------
In new version of BIND9, slave zones are saved by default as 'raw'
format after the zone transfer. Previously the default format was 'text'.
You should use one of the following solutions:
- Remove saved slave zones files so they are saved in the 'raw'
  format when transfered next time.
- Convert zones files to 'raw' format using the 'named-compilezone'
  tool.
- Include 'masterfile-format text;' statement in the slave zone
  definition statement.""")

    return status


################################################################
# These checks can not be run as the rest, as they need to check
# all configuration files at once.

def check_empty_zones_complex():
    """
    Check if there are some zones defined that are now included in empty zones.
    """
    status = EXIT_PASS

    new_ez = ["64.100.IN-ADDR.ARPA",
              "65.100.IN-ADDR.ARPA",
              "66.100.IN-ADDR.ARPA",
              "67.100.IN-ADDR.ARPA",
              "68.100.IN-ADDR.ARPA",
              "69.100.IN-ADDR.ARPA",
              "70.100.IN-ADDR.ARPA",
              "71.100.IN-ADDR.ARPA",
              "72.100.IN-ADDR.ARPA",
              "73.100.IN-ADDR.ARPA",
              "74.100.IN-ADDR.ARPA",
              "75.100.IN-ADDR.ARPA",
              "76.100.IN-ADDR.ARPA",
              "77.100.IN-ADDR.ARPA",
              "78.100.IN-ADDR.ARPA",
              "79.100.IN-ADDR.ARPA",
              "80.100.IN-ADDR.ARPA",
              "81.100.IN-ADDR.ARPA",
              "82.100.IN-ADDR.ARPA",
              "83.100.IN-ADDR.ARPA",
              "84.100.IN-ADDR.ARPA",
              "85.100.IN-ADDR.ARPA",
              "86.100.IN-ADDR.ARPA",
              "87.100.IN-ADDR.ARPA",
              "88.100.IN-ADDR.ARPA",
              "89.100.IN-ADDR.ARPA",
              "90.100.IN-ADDR.ARPA",
              "91.100.IN-ADDR.ARPA",
              "92.100.IN-ADDR.ARPA",
              "93.100.IN-ADDR.ARPA",
              "94.100.IN-ADDR.ARPA",
              "95.100.IN-ADDR.ARPA",
              "96.100.IN-ADDR.ARPA",
              "97.100.IN-ADDR.ARPA",
              "98.100.IN-ADDR.ARPA",
              "99.100.IN-ADDR.ARPA",
              "100.100.IN-ADDR.ARPA",
              "101.100.IN-ADDR.ARPA",
              "102.100.IN-ADDR.ARPA",
              "103.100.IN-ADDR.ARPA",
              "104.100.IN-ADDR.ARPA",
              "105.100.IN-ADDR.ARPA",
              "106.100.IN-ADDR.ARPA",
              "107.100.IN-ADDR.ARPA",
              "108.100.IN-ADDR.ARPA",
              "109.100.IN-ADDR.ARPA",
              "110.100.IN-ADDR.ARPA",
              "111.100.IN-ADDR.ARPA",
              "112.100.IN-ADDR.ARPA",
              "113.100.IN-ADDR.ARPA",
              "114.100.IN-ADDR.ARPA",
              "115.100.IN-ADDR.ARPA",
              "116.100.IN-ADDR.ARPA",
              "117.100.IN-ADDR.ARPA",
              "118.100.IN-ADDR.ARPA",
              "119.100.IN-ADDR.ARPA",
              "120.100.IN-ADDR.ARPA",
              "121.100.IN-ADDR.ARPA",
              "122.100.IN-ADDR.ARPA",
              "123.100.IN-ADDR.ARPA",
              "124.100.IN-ADDR.ARPA",
              "125.100.IN-ADDR.ARPA",
              "126.100.IN-ADDR.ARPA",
              "127.100.IN-ADDR.ARPA",
              ]

    # Create a global config
    configuration = ""
    for fpath, buff in FILES_TO_CHECK:
        configuration += buff + "\n"

    ez_disable_pattern = re.compile("empty-zones-enable\s+no")
    # Check if empty zones are not disabled globally
    found = ez_disable_pattern.findall(configuration)
    if found:
        return status

    # Check new empty zones
    for empty_zone in new_ez:
        pattern = re.compile("zone\s+\"" + empty_zone + "\"", re.IGNORECASE)
        pattern_dis = re.compile(
            "disable-empty-zone\s+\"" + empty_zone + "\"", re.IGNORECASE)
        found = pattern.findall(configuration)
        if found:
            # check if the empty zone is not disabled individually
            found_dis = pattern_dis.findall(configuration)
            if found_dis:
                continue
            status = EXIT_FAIL
            log_high_risk("Found zone \"" + empty_zone + "\" in BIND9 " +
                          "configuration. This zone will be overridden by built-in " +
                          "empty zone if not disabled!")
    
    if status == EXIT_FAIL:
        sol_text.add_solution(
"""Zone declaration that conflicts with built-in empty zones:
----------------------------------------------------------
In new version of BIND9, the list of automatically created empty
zones expanded. Your configuration contains zone that is conflicting
with built-in empty zone. You should use one of following solutions:
- Disable the specific empty zone by using 'disable-empty-zone <zone>;
  statement
- Disable empty zones at globally by using 'empty-zones-enable no;'
  statement.""")

    return status


def check_default_runtime_dir():
    """
    Check if there are statements needed for /var/run -> /run move in 'options'.
    """
    status = EXIT_PASS

    # Create a global config
    configuration = ""
    for fpath, buff in FILES_TO_CHECK:
        configuration += buff + "\n"

    pid_file_pattern = re.compile("pid-file\s+\"\/run\/named\/named\.pid\"")
    session_keyfile_pattern = re.compile("session-keyfile\s+\"\/run\/named\/session\.key\"")
    
    # Check for 'pid-file' statement
    found = pid_file_pattern.findall(configuration)
    if not found:
        ret = fix_pid_file_statement()
        if ret:
            status = EXIT_FIXED
        else:
            log_slight_risk("Did NOT find \"pid-file\" statement in BIND9 configuration.")
            status = EXIT_FAIL

    # Check for 'session-keyfile' statement
    found = session_keyfile_pattern.findall(configuration)
    if not found:
        ret = fix_session_keyfile_statement()
        if ret:
            status = EXIT_FIXED
        else:
            log_slight_risk("Did NOT find \"session-keyfile\" statement in BIND9 configuration.")
            status = EXIT_FAIL
    
    if status == EXIT_FAIL:
        sol_text.add_solution(
"""No 'pid-file' AND/OR 'session-keyfile' statement found:
-------------------------------------------------------
The directory used by named for runtime data has been moved from the BIND
default location, '/var/run/named/', to a new location '/run/named/'.
As a result, the PID file has been moved from the default location
'/var/run/named/named.pid' to the new location '/run/named/named.pid'.
In addition, the session-key file has been moved to '/run/named/session.key'.
These locations need to be specified by statements in the options section.
To fix this you should add:
- 'pid-file  "/run/named/named.pid";' statement into the options section of
  your BIND9 configuration.
- 'session-keyfile  "/run/named/session.key";' statement into the options
  section of your BIND9 configuration.""")
    else:
        sol_text.add_solution(
"""[FIXED] No 'pid-file' AND/OR 'session-keyfile' statement found:
-------------------------------------------------------
The directory used by named for runtime data has been moved from the BIND
default location, '/var/run/named/', to a new location '/run/named/'.
As a result, the PID file has been moved from the default location
'/var/run/named/named.pid' to the new location '/run/named/named.pid'.
In addition, the session-key file has been moved to '/run/named/session.key'.
These locations need to be specified by statements in the options section.
To fix this we added:
- 'pid-file  "/run/named/named.pid";' statement into the options section of
  your BIND9 configuration.
- 'session-keyfile  "/run/named/session.key";' statement into the options
  section of your BIND9 configuration.""")

    return status


#######################################################
### CONFIGURATION CHECKS PART - END
#######################################################
### CONFIGURATION fixes PART - BEGIN
#######################################################

def fix_pid_file_statement():
    """
    Adds 'pid-file' statement into the named.conf
    """
    try:
        new_config = FIXED_CONFIGS[CONFIG_FILE]
    except KeyError:
        with open(CONFIG_FILE, "r") as f:
            new_config = f.read()

    options_pattern = re.compile("options\s+\{(([^\{\}]*)|(\{[^\{\}]*\};)|(.*?))*\};", re.DOTALL)
    matches = re.finditer(options_pattern, new_config)

    match = None
    for m in matches:
        match = m
        break

    if match:
        new_config = new_config[0:match.end()-2] + '\tpid-file "/run/named/named.pid";\n' + new_config[match.end()-2:]
        FIXED_CONFIGS[CONFIG_FILE] = new_config
        return True
    else:
        return False


def fix_session_keyfile_statement():
    """
    Adds 'session-keyfile' statement into the named.conf
    """
    try:
        new_config = FIXED_CONFIGS[CONFIG_FILE]
    except KeyError:
        with open(CONFIG_FILE, "r") as f:
            new_config = f.read()

    options_pattern = re.compile("options\s+\{(([^\{\}]*)|(\{[^\{\}]*\};)|(.*?))*\};", re.DOTALL)
    matches = re.finditer(options_pattern, new_config)

    match = None
    for m in matches:
        match = m
        break

    if match:
        new_config = new_config[0:match.end()-2] + '\tsession-keyfile  "/run/named/session.key";\n' + new_config[match.end()-2:]
        FIXED_CONFIGS[CONFIG_FILE] = new_config
        return True
    else:
        return False


#######################################################
### CONFIGURATION fixes PART - END
#######################################################

def write_fixed_configs_to_disk(result):
    """
    Writes fixed configs in the respective dirs.
    """
    if result > EXIT_FIXED:
        output_dir = os.path.join(VALUE_TMP_PREUPGRADE, "dirtyconf")
        sol_text.add_solution("The config file(s) could not be fixed completely, there are still some issues that need review.")
    else:
        output_dir = os.path.join(VALUE_TMP_PREUPGRADE, "cleanconf")
        sol_text.add_solution("The config file(s) have been completely fixed.")

    for path, buff in FIXED_CONFIGS.iteritems():
        curr_path = os.path.join(output_dir, path[1:])

        # create dirs to make sure they exist
        try:
            os.makedirs(os.path.dirname(curr_path))
        except OSError as e:
            # if the dir already exist (errno 17), pass
            if e.errno == 17:
                pass
            else:
                raise e

        with open(curr_path, "w") as f:
            f.write(buff)
        msg = "Written Fixed config file to '" + curr_path + "'"
        log_info(msg)
        sol_text.add_solution(msg)



def is_config_changed():
    """
    Checks if configuration files changed.
    """
    with open(VALUE_ALLCHANGED, "r") as f:
        files = f.read()
        for fpath, buff in FILES_TO_CHECK:
            found = re.findall(fpath, files)
            if found:
                return True
    return False


def return_with_code(code):
    if code == EXIT_FAIL:
        exit_fail()
    elif code == EXIT_FIXED:
        exit_fixed()
    elif code == EXIT_NOT_APPLICABLE:
        exit_not_applicable()
    elif code == EXIT_PASS:
        exit_pass()
    elif code == EXIT_ERROR:
        exit_error()
    elif code == EXIT_INFORMATIONAL:
        exit_informational()
    else:
        exit_unknown()


def check_user(uid=0):
    """
    Checks if the effective user ID is the one passed as argument.
    """
    if os.geteuid() != uid:
        sys.stdout.write("Need to be root.\n")
        log_error("The script needs to be run under root account")
        exit_error()


def remove_comments(string):
    """
    Removes the following types of comments from passed string and returns it:
    // .*
    # .*
    /* (.|\n)* */
    """
    pattern = "(\/\*(.|\n)*\*\/)|(#.*\n)|(\/\/.*\n)"
    replacer = ""
    return re.sub(pattern, replacer, string)


def is_file_loaded(path=""):
    """
    Checks if file with given 'path' is already loaded in FILES_TO_CHECK.
    """
    for f in FILES_TO_CHECK:
        if f.path == path:
            return True
    return False


def load_included_files():
    """
    Finds configuration files that are included in some configuration
    file, reads it, closes and adds into FILES_TO_CHECK list.
    """
    pattern = re.compile("include\s*\"(.+?)\"\s*;")
    # find includes in all files
    for ch_file in FILES_TO_CHECK:
        includes = re.findall(pattern, ch_file.buffer)
        for include in includes:
            # don't include already loaded files -> prevent loops
            if is_file_loaded(include):
                continue
            try:
                f = open(include, 'r')
            except IOError:
                log_error("Cannot open configuration file: \"" + include +
                          "\"" + "included by \"" + ch_file.path + "\"")
                exit_error()
            else:
                log_info("Include statement found in \"" + ch_file.path + "\": " +
                         "loading file \"" + include + "\"")
                filtered_string = remove_comments(f.read())
                f.close()
                FILES_TO_CHECK.append(ConfFile(buffer=filtered_string,
                                               path=include))


def load_main_config():
    """
    Loads main CONFIG_FILE.
    """
    try:
        f = open(CONFIG_FILE, 'r')
    except IOError:
        log_error(
            "Cannot open the configuration file: \"" + CONFIG_FILE + "\"")
        exit_error()
    else:
        log_info("Loading configuration file: \"" + CONFIG_FILE + "\"")
        filtered_string = remove_comments(f.read())
        f.close()
        FILES_TO_CHECK.append(ConfFile(buffer=filtered_string,
                                       path=CONFIG_FILE))


def main():
    check_user()
    load_main_config()
    load_included_files()
    # need to check also paths of included files
    if not is_config_changed():
        return_with_code(EXIT_PASS)    
    result = run_checks(FILES_TO_CHECK)
    # write the config into the respective dir
    write_fixed_configs_to_disk(result)
    # if there was some issue, write a solution text
    if result > EXIT_PASS:
        solution_file(sol_text.get_text())
    return_with_code(result)


if __name__ == "__main__":
    set_component(COMPONENT)
    main()



#!/usr/bin/python
# -*- Mode: Python; python-indent: 8; indent-tabs-mode: t -*-

import sys, os
import subprocess

ENABLED_SERVICES = "enabled.log"
DISABLED_SERVICES = "disabled.log"
PRESET_FILE = "/usr/lib/systemd/system-preset/90-default.preset"
SYSTEMD_DIR = "/lib/systemd/system"
SYSTEMCTL = "/usr/bin/systemctl"
enabled_services = []
disabled_services = []
preset = []

def open_file(filename):
    try:
        f = open(filename, "r")
        try:
            line = f.readlines()
        except IOError:
            raise
    except IOError:
        raise
    else:
        f.close()
    return line


def run_subprocess(cmd):
    """ wrapper for Popen """
    sp = subprocess.Popen(cmd,
                          stdout=subprocess.PIPE,
                          stderr=subprocess.STDOUT,
                          shell=True,
                          bufsize=1)
    stdout = ''
    for stdout_data in iter(sp.stdout.readline, b''):
        # communicate() method buffers everything in memory, we will read stdout directly
        stdout += stdout_data
        print stdout_data,
    sp.communicate()

    return sp.returncode

def control_service(service, control="enable"):
    cmd = "{0} {1} {2}".format(SYSTEMCTL, control, service)
    print cmd
    run_subprocess(cmd)

def find_service_files(service):
    service_files = (filter(lambda x: x.startswith(service), os.listdir(SYSTEMD_DIR)))
    return service_files

def check_preset(service, control="enable"):
        service_file, extension = service.split('.')
        preset_found = filter(lambda x: service in x, preset)
        if preset_found:
            # If preset contains just <name>.service then enable directory
            if service_file in ''.join(preset_found):
                control_service(service_file, control=control)
            # Found all services for relevant <name>
            else:
                all_service_files = find_service_files(service)
                for service_type in all_service_files:
                    control_service(service_type, control=control)
        else:
            sys.stderr.write("The service %s is not mentioned in 90-default.preset file and therefore postupgrade script won't handle it\n" % service)

def determine_services(services):
    found_services = []
    for service in services:
        service = service.strip()
        if not os.access("{0}/{1}.service".format(SYSTEMD_DIR, service), os.F_OK):
            sys.stderr.write("systemd service %s.service does not exist.\n" % service)
            found_services.extend(find_service_files(service))
        else:
            found_services.append(service+".service")
    return found_services

def enable_services():
    global enabled_services
    try:
        services = open_file(ENABLED_SERVICES)
    except IOError:
        print "Unable to open enabled services"
        sys.exit(1)

    try:
        preset = open_file(PRESET_FILE)
    except IOError:
        print "Unable to open enabled services"
        sys.exit(1)

    enabled_services = determine_services(services)

    for service in enabled_services:
        check_preset(service)

def disable_services():
    global disabled_services
    try:
        services = open_file(DISABLED_SERVICES)
    except IOError:
        print "Unable to open enabled services"
        return

    disabled_services = determine_services(services)

    for service in disabled_services:
        check_preset(service, control="disable")

def main():
    disable_services()
    enable_services()

if __name__ == "__main__":
    sys.exit(main())
